/*******************************************************************************
* Copyright 2007 Seapine Software, Inc. All rights reserved.
*
* IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
* in consideration of your agreement to the following terms, and your use, installation,
* modification or redistribution of this Software constitutes acceptance of these terms.
* If you do not agree with these terms, please do not use, install, modify or redistribute
* this Software.
*
* In consideration of your agreement to abide by the following terms, and subject to these
* terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
* this original Seapine software (the Software), to use, reproduce, modify and redistribute
* the Software, with or without modifications, in source and/or binary forms; provided that
* if you redistribute the Software in its entirety and without modifications, you must retain
* this notice and the following text and disclaimers in all such redistributions of the Software.
* Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
* or promote products derived from the Software without specific prior written permission from
* Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
* implied, are granted by Seapine herein, including but not limited to any patent rights that
* may be infringed by your derivative works or by other works in which the Software may be
* incorporated.
*
* The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
* EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
* OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
*
* IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
* REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
* UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
* IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/
using System;
using System.Collections;
using System.IO;
using System.Text;
using P4COM;

namespace P4JobExport
{
   class P4JobExport
   {
      private const string c_strField101 = "Job";
      private const string c_strField103_Username = "User";
      protected p4 m_p4api = null;
      protected P4User m_p4user = null;
      protected SPSTools.Logger m_log = null;
      protected ArrayList m_listHeadings = null;
      protected ArrayList m_listJobs = null;

      P4JobExport()
      {
         m_p4api = new p4();
         m_log = new SPSTools.Logger("p4jobexport.log");
         m_listHeadings = new ArrayList();
         m_listJobs = new ArrayList();
      }

      // If a field doesn't have a value set on a specific job, it's not included in the results of 
      // 'p4 jobs -l' so we build the list of field names as we parse the entire jobs result set.
      protected void addHeading(string p_strField) { if (!m_listHeadings.Contains(p_strField)) { m_listHeadings.Add(p_strField); } }

      // Each element in the 'p4 jobs -l' array is formatted as '<field name> <field value>'. This 
      // function will split that element into a two-element string array for easier processing.
      public static string[] splitString(string p_str)
      {
         string[] astr = new string[2];
         int i = p_str.IndexOf(' ');
         if (i > 0) { astr[0] = p_str.Substring(0,i); astr[1] = p_str.Substring(i+1, p_str.Length-i-1); }
         else { astr[0] = p_str.Split(' ')[0]; astr[1] = p_str.Split(' ')[1]; }
         return (astr);
      }

      public void runExport(string p_strOutputFile)
      {
         try
         {
            m_p4api.Tagged(); // Tag the output, so it's easier to parse. MUST call before connect!
            m_p4api.Connect();
            m_p4api.ExceptionLevel = 1; // 0 - No exceptions, 1 - Exception on error, 2 - Exception on error or warning.
            m_log.log("Connection successful, proceeding to fetch users");

            m_p4user = new P4User(m_p4api.run("users"));
            m_log.log("User relations created, proceeding to fetch jobs");

            Array aJobs = m_p4api.run("jobs -l");
            m_log.log("Jobs retrieved using 'p4 jobs -l', " + aJobs.Length + " data elements found");

            addHeading(c_strField101); // Field 101, 'Job' cannot be renamed/deleted so add it here to save time.
            long lIndex = 0;
            while (parseJob(aJobs, ref lIndex));
            m_log.log("Jobs data parsed, " + m_listJobs.Count + " jobs found with " + m_listHeadings.Count + " distinct fields");

            StreamWriter output = new StreamWriter(p_strOutputFile.Length > 0 ? p_strOutputFile : ("P4Jobs.csv"), false);
            string strLine = "";
            for (int i = 0; i < m_listHeadings.Count; ++i) { strLine += "," + m_listHeadings[i]; }
            output.WriteLine(strLine.TrimStart(','));
            m_log.log("Job field names written to output");

            for (int i = 0; i < m_listJobs.Count; ++i)
            {
               strLine = ((P4Job)m_listJobs[i]).generateLine(m_listHeadings);
               output.WriteLine(strLine);
            }
            output.Flush();
            output.Close();
            m_log.log("Jobs data written to output");
         }
         catch (Exception p_exception)
         {
            m_log.log("Exporting jobs failed: " + p_exception.Message);
         }
         finally
         {
            m_p4api.Disconnect();
            m_log.close();
         }
      }

      // Parse the job and add it to the array, with field 101 being the start/stop markers.
      protected bool parseJob(Array p_array, ref long p_lIndex)
      {
         if (p_array == null || p_lIndex >= p_array.Length) { return (false); }
         bool fResult = false;
         try
         {
            P4Job job = new P4Job();
            string[] astr = splitString((string)p_array.GetValue(p_lIndex++));
            if (astr[0] == c_strField101)
            {
               m_log.log("Parsing Job #" + astr[1]);
               job.add(astr[0], astr[1].Substring(3)); // strip off the leading 'job' so this can act as the defect #.
            }
            else
            {
               m_log.log("Parser seems to have lost it's way, at index " + (p_lIndex-1));
               return (false);
            }

            for (; p_lIndex < p_array.Length; ++p_lIndex)
            {
               astr = splitString((string)p_array.GetValue(p_lIndex));
               if (astr[0] == c_strField101) { break; } // we've reached the end of the job!
               addHeading(astr[0]);

               string strValue = astr[1];
               if (astr[0] == c_strField103_Username) { strValue = m_p4user.getName(astr[1]); }
               job.add(astr[0], strValue);
            }

            m_log.log("Job parsed successfully!");
            m_listJobs.Add(job);
            fResult = true;
         }
         catch (Exception p_exception)
         {
            m_log.log("Parsing job failed: " + p_exception.Message);
         }
         return (fResult);
      }

      static void Main(string[] p_args)
      {
         P4JobExport p4api = new P4JobExport();
         p4api.runExport(p_args.Length > 0 ? p_args[0] : "");
      }
   }
}
